@@ -22,6 +22,8 @@ from saq.utils import (
 
 logger = logging.getLogger("saq")
 
+ID_PREFIX = "saq:job:"
+
 
 class JobError(Exception):
     def __init__(self, job):
@@ -86,6 +88,9 @@ class Queue:
     def unregister_before_enqueue(self, callback):
         self._before_enqueues.pop(id(callback), None)
 
+    def job_id(self, job_key):
+        return f"{ID_PREFIX}{self.name}:{job_key}"
+
     async def _before_enqueue(self, job):
         for cb in self._before_enqueues.values():
             await cb(job)
@@ -260,27 +265,28 @@ class Queue:
                     logger.info("Sweeping job %s", job)
         return swept
 
-    async def listen(self, job_ids, callback, timeout=10):
+    async def listen(self, job_keys, callback, timeout=10):
         """
         Listen to updates on jobs.
 
-        job_ids: sequence of job ids
+        job_keys: sequence of job keys
         callback: callback function, if it returns truthy, break
         timeout: if timeout is truthy, wait for timeout seconds
         """
         pubsub = self.redis.pubsub()
-
+        job_ids = [self.job_id(job_key) for job_key in job_keys]
         await pubsub.subscribe(*job_ids)
 
         async def listen():
             async for message in pubsub.listen():
                 if message["type"] == "message":
                     job_id = message["channel"].decode("utf-8")
+                    job_key = Job.key_from_id(job_id)
                     status = Status[message["data"].decode("utf-8").upper()]
                     if asyncio.iscoroutinefunction(callback):
-                        stop = await callback(job_id, status)
+                        stop = await callback(job_key, status)
                     else:
-                        stop = callback(job_id, status)
+                        stop = callback(job_key, status)
 
                     if stop:
                         break
@@ -301,7 +307,11 @@ class Queue:
         await self.redis.set(job.id, self.serialize(job))
         await self.notify(job)
 
-    async def job(self, job_id):
+    async def job(self, job_key):
+        job_id = self.job_id(job_key)
+        return await self._get_job_by_id(job_id)
+
+    async def _get_job_by_id(self, job_id):
         async with self._op_sem:
             return self.deserialize(await self.redis.get(job_id))
 
@@ -384,7 +394,7 @@ class Queue:
                 "BLMOVE", self._queued, self._active, "RIGHT", "LEFT", timeout
             )
         if job_id is not None:
-            return await self.job(job_id)
+            return await self._get_job_by_id(job_id)
 
         logger.debug("Dequeue timed out")
         return None
@@ -505,23 +515,23 @@ class Queue:
             }
             for kw in iter_kwargs
         ]
-        job_ids = [Job.id_from_key(key["key"]) for key in iter_kwargs]
-        pending_job_ids = set(job_ids)
+        job_keys = [key["key"] for key in iter_kwargs]
+        pending_job_keys = set(job_keys)
 
-        async def callback(job_id, status):
+        async def callback(job_key, status):
             if status in TERMINAL_STATUSES:
-                pending_job_ids.discard(job_id)
+                pending_job_keys.discard(job_key)
 
             if status in UNSUCCESSFUL_TERMINAL_STATUSES and not return_exceptions:
                 return True
 
-            if not pending_job_ids:
+            if not pending_job_keys:
                 return True
 
         # Start listening before we enqueue the jobs.
         # This ensures we don't miss any updates.
         task = asyncio.create_task(
-            self.listen(pending_job_ids, callback, timeout=timeout)
+            self.listen(pending_job_keys, callback, timeout=timeout)
         )
 
         try:
@@ -534,7 +544,7 @@ class Queue:
 
         await asyncio.wait_for(task, timeout=timeout)
 
-        jobs = await asyncio.gather(*[self.job(job_id) for job_id in job_ids])
+        jobs = await asyncio.gather(*[self.job(job_key) for job_key in job_keys])
 
         results = []
         for job in jobs:
